@@ -1,5 +1,7 @@
+use std::ptr::NonNull;
+
 use crate::{pattern::Pattern, FaitheError};
-use windows::Win32::System::ProcessStatus::MODULEINFO;
+use windows::Win32::{System::{ProcessStatus::MODULEINFO, LibraryLoader::LoadLibraryW}, Foundation::PWSTR};
 
 /// Basic information about process's module.
 #[derive(Debug, Clone)]
@@ -22,25 +24,39 @@ impl From<MODULEINFO> for ModuleInfo {
     }
 }
 
+/// Either loads library from path or returns an address of already existing module.
+pub fn load_library(lib_name: impl AsRef<str>) -> crate::Result<NonNull<()>> {
+    unsafe {
+        let utf16 = format!("{}\x00", lib_name.as_ref()).encode_utf16().collect::<Vec<_>>();
+        let lib = LoadLibraryW(PWSTR(utf16.as_ptr()));
+        if lib.is_invalid() {
+            Err(FaitheError::last_error())
+        } else {
+            Ok(NonNull::new_unchecked(lib.0 as _))
+        }
+    }
+}
+
 /// Returns an address(its handle) of the module.
 /// Always returns Ok(address).
 #[cfg(feature = "nightly")]
 pub fn get_module_address(mod_name: impl AsRef<str>) -> crate::Result<*mut ()> {
-    use std::ptr::null_mut as null;
-
+    use crate::{containing_record, internal::LdrDataTableEntry};
     use super::get_peb;
 
     unsafe {
-        let first = get_peb().ldr_data.in_memory_order_links.blink;
+        let first = containing_record!(get_peb().ldr_data.in_memory_order_links, LdrDataTableEntry, in_memory_order_links);
         let mut current = first;
         loop {
-            if (*current).base_dll_name.decode_utf16() == mod_name.as_ref() {
-                break Ok((*current).dll_base as _);
+            if let Some(s) = (*current).base_dll_name.decode_utf16() {
+                if s == mod_name.as_ref()  {
+                    break Ok((*current).dll_base as _);
+                }
             }
-
-            current = (*current).in_memory_order_links.blink;
+            
+            current = containing_record!((*current).in_memory_order_links, LdrDataTableEntry, in_memory_order_links);
             if current == first {
-                break Ok(null());
+                break Err(FaitheError::ModuleNotFound);
             }
         }
     }
@@ -68,21 +84,24 @@ pub fn get_module_address(mod_name: impl AsRef<str>) -> crate::Result<*mut ()> {
 /// Returns information about the specified module.
 #[cfg(feature = "nightly")]
 pub fn get_module_information(mod_name: impl AsRef<str>) -> crate::Result<ModuleInfo> {
+    use crate::{containing_record, internal::LdrDataTableEntry};
     use super::get_peb;
 
     unsafe {
-        let first = get_peb().ldr_data.in_memory_order_links.blink;
+        let first = containing_record!(get_peb().ldr_data.in_memory_order_links, LdrDataTableEntry, in_memory_order_links);
         let mut current = first;
         loop {
-            if (*current).base_dll_name.decode_utf16() == mod_name.as_ref() {
-                break Ok(ModuleInfo {
-                    dll_base: (*current).dll_base,
-                    image_size: (*current).image_size as _,
-                    entry_point: (*current).entry_point,
-                });
+            if let Some(s) = (*current).base_dll_name.decode_utf16() {
+                if s == mod_name.as_ref()  {
+                    break Ok(ModuleInfo {
+                        dll_base: (*current).dll_base,
+                        image_size: (*current).image_size as _,
+                        entry_point: (*current).entry_point,
+                    });
+                }
             }
 
-            current = (*current).in_memory_order_links.blink;
+            current = containing_record!((*current).in_memory_order_links, LdrDataTableEntry, in_memory_order_links);
             if current == first {
                 break Err(FaitheError::ModuleNotFound);
             }
